#!/usr/bin/perl -w
#
# areasbbsadm.pl - v0.1 
# Administration of the areasbbs or fareasbbs tables
# - Createing an areasbbs table in an SQL (Mysql) based database.
# - Createing an fareasbbs table in an SQL (Mysql) based database.
# - Adding descriptions to either table from a file (not yet implemented)
#  Copyright (c) 2001-2002 Robert James Clay.  All Rights Reserved.
#  This is free software;  you can redistribute it and/or
#  modify it under the same terms as Perl itself.

use strict;
use Getopt::Std;
use vars qw/ $opt_h $opt_a $opt_d $opt_f $opt_l $opt_v $opt_x /;

use FTN::Utility qw(&logging);

my ($dbh, $sql_stmt, $Debug, $descriptionfile, $Logfile);

#my $progid = "abadm";

getopts('d:l:afhvx');

if($opt_h) {
    print "Usage: areasbbsadm.pl [-f] [-l STDOUT|logfile] [-v] [-x]...\n";
    print "  -a  create areasbbs table, or...\n";
    print "  -f  create file areasbbs table...\n";
    print "  -d  description file pathname .\n";
    print "  -l  stdout or logfile pathname .\n";
    print "  -v  Verbose \n";
    print "  -x  Set debug...\n\n";
    exit;
}

if ($opt_l) {
    $Logfile=$opt_l;  # this needs to be the filename & path
    undef $opt_l;
} else {
    $Logfile="stdout";		# defaults to standard out.
}

if ($opt_d) {
    $descriptionfile=$opt_d;  # this needs to be the filename & path
    undef $opt_d;
} else {
    $descriptionfile="echodesc.txt";		# defaults to echodesc.txt.
}

if ($opt_v) {
    print "logfile is $Logfile.\n";
    &log("Verbose flag is set ...");
}
	
if ($opt_x) {
    $Debug = $opt_x;	# set debug flag
    undef $opt_x;
    &log("Debug flag is set ...");
} else {
    $Debug = 0;
}



if ($opt_a) {
    &createareastable();

} elsif ($opt_f) {
    &createareastable();

} elsif ($descriptionfile) {
    &UpdateAreaDescriptions();
}

#  
exit();

############################################
# create areasbbs or fareasbbs table
############################################
sub createareastable {

# connect to database
    &openftndb();	# open message database

## drop the old echo table, if it exists.
## uncomment this if needed.
#    if ($opt_f) {
#        $sql_stmt = "DROP TABLE fareasbbs";
#    } else {
#	$sql_stmt = "DROP TABLE areasbbs";
#    }
#
#    print " $sql_stmt ";
#    $dbh->do( "$sql_stmt " ) or  die $DBI::errstr;
#        or $drop_ok = "0"; 

# build Create Table sql statement
    if ($opt_f) {
        $sql_stmt = "CREATE TABLE fareasbbs( ";
    } else {
        $sql_stmt = "CREATE TABLE areasbbs( ";
    }

    $sql_stmt .= "id              INT NOT NULL AUTO_INCREMENT PRIMARY KEY, ";

# areaname     name as distributed
    $sql_stmt .= "areaname        VARCHAR(32) DEFAULT '' NOT NULL, ";

# bbsname      name of table in MySQL database
    $sql_stmt .= "bbsname         VARCHAR(32) DEFAULT '' NOT NULL, ";

#  description    
    $sql_stmt .= "description     VARCHAR(128) DEFAULT '' NOT NULL, ";

# prinode
    $sql_stmt .= "prinode         VARCHAR(72) DEFAULT ' ' NOT NULL, ";

# prinode
    $sql_stmt .= "uplink          VARCHAR(72) DEFAULT ' ' NOT NULL, ";

#  domain       
    $sql_stmt .= "domain          VARCHAR(8) DEFAULT 'fidonet' NOT NULL, ";

    if ($opt_f) {	# do fareasbbs table specific fields
#  file system path where files for this file echo are put
        $sql_stmt .= "dirpath     VARCHAR(128) DEFAULT '' NOT NULL, ";

    } else {	# do areasbbs table specific fields
#  maxmsgs    Maximum number of msgs to keep                   
        $sql_stmt .= "maxmsgs     MEDIUMINT DEFAULT '100' NOT NULL, ";

#  maxdays    Maximum number of days to keep msgs
        $sql_stmt .= "maxdays     TINYINT DEFAULT '28' NOT NULL, ";
#  no real boolean type?
#  type  L=local, N=netmail, C=echo message area	
        $sql_stmt .= "type        CHAR(1) DEFAULT 'L' NOT NULL, ";
    }	

#   What to do for the index?
    if ($opt_f) {
        $sql_stmt .= "INDEX fareasbbsidx (id, areaname) ";
    } else {
        $sql_stmt .= "INDEX areasbbsidx (id, areaname) ";
    }

    $sql_stmt .= ") ";

#print " $sql_stmt ";
    $dbh->do( "$sql_stmt " ) or die $DBI::errstr;

# disconnect from database
    &closeftndb();

}

###################################################
# Update area descriptions in (f)areas bbs table
###################################################
sub UpdateAreaDescriptions {
# This subroutine adds conference (echo) descriptions from
# a file to the areasbbs table.  

    my ($distributionname, $desc); 
    
    open (DESCFILE, $descriptionfile)
        or die &log("Cannot open Echo Description file:  $descriptionfile");
      
# connect to database
    &openftndb();	# open message database

#   this is the begining of the read file loop
    while(<DESCFILE>) {

        ($distributionname,$desc) = split(' ', $_, 2);
      
	$distributionname =~ tr/\"/ /;  # remove qoutes from name
	$distributionname =~ tr/\'/ /;  # remove qoutes from name
	
        chomp($desc);	# remove CR
#	$desc =~ tr/\"/\'/;  # change double quotes to single

	if ($opt_v) {&log("updateing description for $distributionname.")};

        $sql_stmt = "UPDATE areasbbs ";

#        $sql_stmt .= "SET description='$desc' ";

        $sql_stmt .= "SET description= ";


        $sql_stmt .= $dbh->quote($desc);	#

        $sql_stmt .= " WHERE areaname = '$distributionname' ";

        if($opt_x) {

		print "SQL Statement:  $sql_stmt \n";
        }

    $dbh->do( "$sql_stmt " );

#  this is the end of the read file loop
}

# disconnect from database
    &closeftndb();

    close DESCFILE;

}
############################################
# open FTN MySQL database for operations
############################################
sub openftndb {
# Open message database
#   Assumes that $dbname, $dbuser, & $dbpass
# have been set.  $dbuser must already 
# have the priveledges to create a table.

    if ($opt_v) {&log("Opening FTN database")};
	
    my $dbname = 'ftndbtst';
    my $dbuser = 'sysop';
    my $dbpass = 'ftntstpw';
 
    use DBI;

    ( $dbh = DBI->connect("dbi:mysql:dbname=$dbname", $dbuser, $dbpass) )
	or die &log($DBI::errstr);

}

############################################
# Close FTN 
############################################
sub closeftndb {
#
    if ($opt_v) {&log("Closing FTN database")};

    ( $dbh->disconnect )
	or die &log($DBI::errstr);
	
}

#############################################
#  logging subrouting.  requires FTN::Utility
#############################################
sub log {
#
    my(@text) = @_;
    my $progid="abadm";

    &logging($Logfile, $progid, @text);

}
############################################
