#!/usr/bin/perl -w
#
# areatable.pl - v0.1 
# Createing an conference area table in an SQL (Mysql) based database.
#  Copyright (c) 2001-2002 Robert James Clay.  All Rights Reserved.
#  This is free software;  you can redistribute it and/or
#  modify it under the same terms as Perl itself.

# note that the description option does not work for more than the first word...
# use some other way of initially putting description in?

use strict;
use Getopt::Std;
use vars qw/ $opt_a $opt_b $opt_l $opt_t $opt_d $opt_f $opt_h $opt_v $opt_x /;

my ($area, $bbsname, $desc, $type, $domain, $dbh, $sql_stmt, $Logfile, $Debug);
 
#print @INC"\n";

getopts('a:b:t:f:d:l:vxh');

if($opt_h) {
    print "\nUsage: areatable.pl -a echoname -b bbsname [-t L|N|C] [-l logfile] [-d domain] [-f descriptionfile] [-x]\n";
    print "echoname = areaname as distributed...\n";
    print "bbsname = name of echoarea to be added to bbs, defaults to same as echoname..\n";
    print "type = L=local, N=netmail, C=Conference. defaults to C\n";
    print "[-d domain] = if not present, domain defaults to table default (fidonet). \n\n";
    print "[-f descriptionfile] = if not present, echo description defaults to name of echo. \n\n";
    print "[-x] if present, debug mode is set...\n";
    exit;
}

if ($opt_v) {
    &log("Verbose flag is set ...");
}
	
if ($opt_x) {
    $Debug = $opt_x;	# set debug flag
    undef $opt_x;
    print "\nDebug flag is set ...\n";
} else {
    $Debug = 0;
}

if ($opt_l) {
    $Logfile = $opt_l;	# set log path 
    undef $opt_l;
} else {
    $Logfile = "areasadm.log";  # in current dir
}

if ($opt_a) {
    $area=$opt_a;   # ? test for a period in echo name (mysql doesn't allow)
		    #so if a period is present it needs to be changed to 
		    #something like an underscore & the bbsname/table name
		    #changed appropriately.  If $opt_e not set, then will
		    #simply remove the period from the name.

    undef $opt_a;
} else {
    $area="TESTECHO";
}

if ($opt_b) {
    $bbsname=$opt_b;
    undef $opt_b;
} else {
    $bbsname=$area;    # bbsname defaults to same as echoname
}

if ($opt_f) {	#  now just defaults to same as echoname
    $desc=$area;    # change this part to pull description from a file
} else {
    $desc=$area;    # description defaults to same as echoname
}

if ($opt_d) {
    $domain=$opt_d;    # setting domain name
} else {
    $domain="";    # domain defaults to blank, which defaults to default
		   # setting of table field, usually 'fidonet'. 
}

if ($opt_t) {
    $type=$opt_t;
} else {
    $type="C";    # defaults to C, i.e. echo/conference
}



# connect to database
#   Assumes that $dbname, $dbuser, & $dbpass
# have been set.  $dbuser must already 
# have the priveledges to create a table.

my $dbname = 'ftndbtst';
my $dbuser = 'sysop';
my $dbpass = 'ftntstpw';
 
use DBI;

( $dbh = DBI->connect("dbi:mysql:dbname=$dbname", $dbuser, $dbpass) )
    or die $DBI::errstr;


## drop the old echo table, if it exists.
##   need a check for if it exists first...
#$sql_stmt = "DROP TABLE ";
#$sql_stmt .= $bbsname;

##print " $sql_stmt \n";
#$dbh->do( "$sql_stmt " );
##    or $drop_ok = "0"; 

# build Create Table sql statement
$sql_stmt = "CREATE TABLE ";
$sql_stmt .= $bbsname;
$sql_stmt .= "( ";

$sql_stmt .= "id                            INT NOT NULL AUTO_INCREMENT PRIMARY KEY, ";

# ftscdate
$sql_stmt .= "ftscdate                      VARCHAR(20) DEFAULT '' NOT NULL, ";

# datetime                      
$sql_stmt .= "datetime                      TIMESTAMP(14) NOT NULL, ";

# fromnode               
$sql_stmt .= "fromnode                      VARCHAR(72) DEFAULT '' NOT NULL, ";
# tonode                 
$sql_stmt .= "tonode                        VARCHAR(72) DEFAULT '' NOT NULL, ";

#  fromname              
$sql_stmt .= "fromname                      VARCHAR(36) DEFAULT '' NOT NULL, ";

#  toname                
$sql_stmt .= "toname                        VARCHAR(36) DEFAULT '' NOT NULL, ";

#  subject               
$sql_stmt .= "subject                       VARCHAR(72) DEFAULT '' NOT NULL, ";

#  attributes                    
$sql_stmt .= "attrib                        SMALLINT UNSIGNED DEFAULT '0' NOT NULL, ";

#  msgeid                        
$sql_stmt .= "msgid                         VARCHAR(72) DEFAULT '' NOT NULL, ";

#  replyid                       
$sql_stmt .= "replyid                       VARCHAR(72) DEFAULT '' NOT NULL, ";

#  no real boolean type?
#  local     Yes or No for if locally entered
$sql_stmt .= "local                         CHAR(1) DEFAULT 'Y' NOT NULL, ";

#  rcvd     Yes or No for if rcvd.  How it is used depends on the table.
$sql_stmt .= "rcvd                          CHAR(1) DEFAULT 'N' NOT NULL, ";

#  sent     Yes or No for if scanned out yes
$sql_stmt .= "sent                          CHAR(1) DEFAULT 'N' NOT NULL, ";


#  body                  
#$sql_stmt .= "body                          MEDIUMBLOB DEFAULT '[empty message]' NOT NULL, ";
$sql_stmt .= "body                          MEDIUMBLOB, ";


#   What to do for the index?
$sql_stmt .= "INDEX echomsgidx (id, msgid, replyid) ";

$sql_stmt .= ") ";

#print " $sql_stmt \n";
$dbh->do( "$sql_stmt " ) or die $DBI::errstr;

#
# update areasbbs table
#   add code check if area entry is present first
 $sql_stmt = "DELETE FROM areasbbs ";
    $sql_stmt .= "WHERE areaname = '$area' ";

#print " $sql_stmt \n";
    $dbh->do( "$sql_stmt " );
#        or $insert_ok = "0"; 

    $sql_stmt = "INSERT INTO areasbbs ";

    if ($opt_d) {
        $sql_stmt .= "(areaname,bbsname,type,domain,description) ";
    } else {
        $sql_stmt .= "(areaname,bbsname,type,description) ";
    }
    $sql_stmt .= "VALUES (";
#				  need to add the rest?
    $sql_stmt .= "'$area', ";
    $sql_stmt .= "'$bbsname', ";
    $sql_stmt .= "'$type', ";
    if ($opt_d) {
        $sql_stmt .= "'$domain', ";
    }
    $sql_stmt .= "'$desc') ";

#print " $sql_stmt \n";
    $dbh->do( "$sql_stmt " ) or die $DBI::errstr;

#
# disconnect from database

( $dbh->disconnect )
	or  die $DBI::errstr;

exit();

#############################################
#  logging subrouting.  requires FTN::Utility
#############################################
sub log {
#
    my(@text) = @_;
    my $progid="MTBLE";

    &logging($Logfile, $progid, @text);

}

############################################
# open FTN MySQL database for operations
############################################
sub openftndb {
# Open message database
#   Assumes that $dbname, $dbuser, & $dbpass
# have been set.  $dbuser must already 
# have the priveledges to create a table.

    if ($opt_v) {&log("Opening FTN database")};
	
    my $dbname = 'ftndbtst';
    my $dbuser = 'sysop';
    my $dbpass = 'ftntstpw';
 
    use DBI;

    ( $dbh = DBI->connect("dbi:mysql:dbname=$dbname", $dbuser, $dbpass) )
	or die &log($DBI::errstr);

}

############################################
# Close FTN 
############################################
sub closeftndb {
#
    if ($opt_v) {&logging("Closing FTN database")};

    ( $dbh->disconnect )
	or die &log($DBI::errstr);
	
}

############################################
