#!/usr/bin/perl -w
#
# bbs2txt.pl - v0.1 
# Perl script that extracts a message from the bbs db & puts
# it to a text file.
#  Copyright (c) 2001-2002 Robert James Clay.  All Rights Reserved.
#  This is free software;  you can redistribute it and/or
#  modify it under the same terms as Perl itself.

# Setup
use strict;
use Getopt::Std;
use vars qw/ $opt_a $opt_n $opt_o $opt_l $opt_x $opt_h /;
use FTN::Utility qw(&logging);


my ($msgarea, $datetime, $from, $OUTFILE, $Debug, 
    $to, $subj, $msgbody, $logfile,  $dbh, $sth, 
#  not yet using these
#    $msgid, $replyid,$fromnode, $tonode, $attrib,  
    $MsgBody, $msgnumber, $logfile, $pagespec);

my $progid = "b2txt";


# How Wide (in characters) to make messages
my $wrap = "80";

# No user serviceable parts inside

&commandline();	#  get command line parameters

&exportmsg();  #  export msg to a text file

############################################
# subroutines
############################################

############################################
# Check for command line arguments, handle.
############################################
sub commandline {

    getopts('a:n:o:l:xh');

    if ($opt_h) {
        &help ();	# print help	
	exit(1);	# & exit
    }

    if ($opt_x) {
        $Debug = $opt_x;	# set debug flag
        undef $opt_x;
        print "\nDebug flag is set ...\n";
    } else {
        $Debug = 0;
    }
	
    if ($opt_a) {
	$msgarea=$opt_a;
	undef $opt_a;
    } else {
        $msgarea="TESTECHO";   # default message area name
    }

    if ($opt_n) {
	$msgnumber=$opt_n;
	undef $opt_n;
    } else {
        $msgnumber="1";   # default message number is 1
    }

    if ($opt_o) {
        $OUTFILE=$opt_o;  # this needs to be the filename & path
        undef $opt_o;
    } else {
        $OUTFILE="message.txt";		# default outbound filename
    }

    if ($opt_l) {
        $logfile=$opt_l;  # this needs to be the filename & path
        undef $opt_l;
    } else {
        $logfile="bbsdbpl.log";		# default log file
    }

}

############################################
# Help
############################################
sub help {
    print "\nUsage:  bbs2txt.pl -a AreaName -n MessageNumber -o OutFileName [-l logfilename] [-x] \n\n";
    print "   -a   Message Area Name. \n";
    print "   -n   Message Number. \n";
    print "   -o   Filename & path that msg is to be written to. \n";
    print "   -l   Log filename (defaults to bbsdbpl.log)\n";
    print "   -x   Debug Mode\n";
    print "\n";
}

############################################
# Export msg by msg number & area name
############################################
sub exportmsg {
    # export msg
	
    if ($Debug) {print "Exporting message $msgnumber in $msgarea.\n"};	# for testing

    &getmsgfromdb;
	
    if ($#lines < 0) {
       @lines = ("[empty message]");
    }
    
# wrap msg body to 80 lines max
    $MsgBody = &wraparray(@lines); 
        
    # What output format are we using?
    &printtxt;		# defaults to outputing text

    &logging("Completed exporting msg $msgnumber in $msgarea."); 
		
}

############################################
# Get Message from database
############################################

sub getmsgfromdb {

    my ($rv, $rc, $sql_stmt, @row, $row);

    &openftndb();	# connect to database

#   need to test first for a period in the $msgarea name
      
#  get msg from database
    $sql_stmt = "SELECT  ";
    $sql_stmt .= "ftscdate,fromname,toname,";
    $sql_stmt .= "subject,body FROM ";
    $sql_stmt .= $msgarea;
    $sql_stmt .= " WHERE id='$msgnumber'";

    if ($Debug) {print "SQL= $sql_stmt.\n"};

# prepare sql statement for execution
    $sth = $dbh->prepare( "$sql_stmt " )
	or die "Can't prepare $sql_stmt:  dbh->errstr.\n";

#  execute sql statment
    $rv = $sth->execute
	or die "Can't execute the query: $sth->errstr";

    if ($Debug) {print "return value from execute is $rv.\n"};

#  get msg info into correct variables for later use

    while(@row = $sth->fetchrow_array) {
	$datetime =$row[0];
	$from = $row[1];
#	    if ($Debug) {print "From= $from.\n"};
	$to = $row[2];
	$subj = $row[3];
	$MsgBody = $row[4];
    }    

#  finished with the statement handle
    $rc = $sth->finish;

    if ($Debug) {print "return value from finish is $rc.\n"};
    
    &closeftndb();	# disconnect from database
#    ( $dbh->disconnect ) or die $DBI::errstr;


}

############################################
# Print text format messages
############################################

sub printtxt {

    $pagespec = getData( "TEXT" );
	
    &addcontent;
	
    open(MSGFILE, ">$OUTFILE") ||
    &logging("Creating msg file $OUTFILE",$!);
    binmode(MSGFILE);	# not needed for all systems...
      
    print MSGFILE "$pagespec";

    close(MSGFILE);
#    undef @lines;    # is this needed?
}

############################################
# Replace placeholders.
############################################

sub addcontent {

    $pagespec =~ s/<!-- AREA -->/$msgarea/s;
    $pagespec =~ s/<!-- MSG -->/$msgnumber/s;
    $pagespec =~ s/<!-- FROM -->/$from/s;
    $pagespec =~ s/<!-- TO -->/$to/s;
    $pagespec =~ s/<!-- SUBJECT -->/$subj/s;
    $pagespec =~ s/<!-- DATETIME -->/$datetime/s;
    $pagespec =~ s/<!-- MESSAGE -->/$MsgBody/s;	
#    $pagespec =~ s/<!-- MAILER -->/$mailer/s if ($mailer);
#    $pagespec =~ s/<!-- ORIGIN -->/$origin/s;
#    $pagespec =~ s/<!-- SEENBY -->/$seenby/s if ($seenby);
#    $pagespec =~ s/<!-- KLUDGES -->/$kludges/s;
}

#############################################
## Wrap long lines of text  -  is this even needed?
#############################################

#sub wraparray {
#   my (@temp);
#   while ($#_ > -1) {
#      $_ = shift;
#      while (length($_) > $wrap) {
#         if ($_ =~ /.{1,$wrap}\s/) {
#            push(@temp,$&);
#            $_ = $';
#         }
#         else {
#            last;
#         }
#      }
#      push(@temp,$_);
#   }
#   @temp;
#}

############################################
# openftndb
############################################
sub openftndb {
# Open message database
#   Assumes that $dbname, $dbuser, & $dbpass
# have been set.  $dbuser must already 
# have the priveledges to create a table.

    if ($opt_v) &logging($logfile, $progid, "Opening Msg database");
	
    my $dbname = 'ftndbtst';
    my $dbuser = 'sysop';
    my $dbpass = 'ftntstpw';
 
    use DBI;

    ( $dbh = DBI->connect("dbi:mysql:dbname=$dbname", $dbuser, $dbpass) )
	or die &logging($logfile, $progid, $DBI::errstr);

}

############################################
# closeftndb
############################################
sub closeftndb {
#
    if ($opt_v) &logging($logfile, $progid, "Closing Msg database");

    ( $dbh->disconnect )
	or die &logging($logfile, $progid, $DBI::errstr);
	
}


# --------------------------------------------------------------------
# Function:  getData
# Returns:  section as a String, or undef if no section of the
#   provided name is found.
# Parameters:
#   $section    String with desired section name
# Description:  Allows for named sections of the __DATA__ block of this
#   script to be accessed.
# --------------------------------------------------------------------
{
my %data;
sub getData {
    my ( $section ) = @_;
    if ( ! %data ) {
    	my $block = '';
	while ( 1 ) {
	    my $line = <DATA>;
	    if ( !defined( $line ) or $line =~ m/^\s*$/ ) {
		if ( $block ne '' ) {
		    $block =~ s/\\\n\s*//gs;
		    $block =~ s/^([A-Z_]+):(.*)$/$2/s;
		    $data{$1} = $block;
		    $block = ''
		}
		    last if ( !defined( $line ) );
	    } else {
	    	$block .= $line;
	}
    }
    close( DATA );
}
return ( defined( $data{$section} ) ? $data{$section} : undef );
}
}

#####################################################################
# DATA SECTION
#####################################################################
__DATA__

TEXT:
Area: <!-- AREA -->  Msg: <!-- MSG -->
From: <!-- FROM --> 
To:   <!-- TO --> 
Subject: <!-- SUBJECT -->
Date: <!-- DATETIME -->  
=====================================================================
<!-- MESSAGE -->

#  not using these...
<!-- MAILER -->
<!-- ORIGIN -->
<!-- SEENBY -->
<!-- KLUDGES -->

