#!/usr/bin/perl -w
#
# bbsdbpladm.pl - v0.0
# This is a script used to perform maintenance on the message tables
# created by the pkt2txt.pl script and others.
#  Delete messages by date or by a max number of msgs per message area.
#  . 
# Copyright (c) 2002 Robert James Clay. 
# All Rights Reserved. This is free software;  you can redistribute
# it and/or modify it under the same terms as Perl itself.

# Setup

use strict;
#use File::Copy;  <- will I need this?
use Getopt::Std;

use vars qw/ $opt_p $opt_a $opt_n $opt_d $opt_l $opt_v $opt_x $opt_h /;

use FTN::Utility qw(&logging);


my ($PACKAREA, $areas, $VERBOSE, $dbh,
    $DEBUG, %areas, $progid, $logfile);
 

my $progid = "bdbadm";
my $TEMPDIR = "/tmp/bdbadm.$$";		# needed?

# No user serviceable parts inside

# &commandline();
getopts('p:d:n:a:b:l:vxh');

if ($opt_h) {
    &help ();
    exit(1);
}

if ($opt_l) {
    $logfile=$opt_l;  # this needs to be the filename & path
    undef $opt_l;
} else {
    $logfile="bbsdbpladm.log";		# default log file
}

if ($opt_x) {
    $DEBUG = $opt_x;	# set debug flag
    undef $opt_x;
    print "\nDebug flag is set ...\n";
} else {
    $DEBUG = 0;
}
	
if ($opt_v) {
    $VERBOSE = $opt_v;	# set debug flag
    undef $opt_v;
    print "\nVerbose flag is set ...\n";
} else {
    $VERBOSE = 0;
}
	
if ($opt_a) {
    $PACKAREA=$opt_a;  # Areaname (= dir name).
    undef $opt_a;
    &areamaintenance;
} else {
    &allareasmaintenance;	# defaults to All echos
}


# do any neccessary cleanup prior to exiting.

exit;

############################################
# subroutines
############################################

############################################
# allareasmaintenance
############################################
sub allareasmaintenance {
#    $PACKAREA="all";	# defaults to All echos
#  get list of message areas & then call areamaintenance
# with $PACKAREA set to the name of each one.

    my ($sth, $i);
    
    &logging($logfile, $progid, "Maintenance of all areas selected");

    &openmsgdb();	# open message database
    
    $sql_stmt = "SELECT bbsname FROM areasbbs WHERE type = 'C' ";

    if ($DEBUG) &logging($logfile, $progid, "$sql_stmt");
    #
    $sth->prepare( "$sql_stmt " )
	or die &logging($logfile, $progid, $sth->errstr); 
    $sth->execute( "$sql_stmt " )
	or die &logging($logfile, $progid, $sth->errstr); 

    my @areas = $sth->fetchall_arrayref
	or die &logging($logfile, $progid, $sth->errstr); 

    $sth->finish();	# done with sql statement
    
    &closeftndb();	# close msg database
    
    if (@areas) == 0 {	# if areas array is empty, then exit
        &logging($logfile, $progid, "No conference type areas to pack.");
        exit();
    } else { 	# pack areas
        for $i ( 0 .. $#{$areas} )  {
	    $PACKAREA = $areas->[$i];
	    &areamaintenance;	# pack area $PACKAREA
	}
    }


}

############################################
# areamaintenance
############################################
sub areamaintenance {
    # assumes $PACKAREA variable holds area name 

    # if pack option set, check which type of packing is to be done
    # and call the appropriate subroutine.  Defaults to no packing.
    if ($opt_p) {

	&openftndb();
	
        if ($opt_p == "max") {
    	    # Pack to a maximum of opt_n message files per area
	    &packbymaxmsgs();
    
        } elseif ($opt_p == "age") {
	    # Pack by opt_d days old.
	    &packbydays();
	}

	&closeftndb();
	
    }	# end opt_p 'if defined' statement

}
############################################
# pack by maximum number of msgs
############################################
sub packbymaxmsgs {
# Pack message directorys to a maximum of
# opt_n message files per area. defaults to # of msgs set for
# msg area in areasbbs table.

    my ($sql_stmt, $maxmsgs, $msgcount);

    if ($opt_n) {
	$maxmsgs = $opt_n;
    } else 
    # this will later check the areasbbs table entry for the maxmsgs field,
    # if that is not zero, then $daysold gets set to it.  if zero, return.
        &logging($logfile, $progid, "Can't pack $PACKAREA; max # of msgs not set ");
	return;
    }

    # assumes $PACKAREA variable holds area name 


    $sql_stmt = "SELECT COUNT(*) FROM $PACKAREA ";

    if ($DEBUG) &logging($logfile, $progid, "$sql_stmt");
    #
    $msgcount = $dbh->do( "$sql_stmt " )
	or die &logging($logfile, $progid, $sth->errstr); 

    if ($VERBOSE) &logging($logfile, $progid, "Area $PACKAREA has $msgcount messages");

    &logging($logfile, $progid, "Packing by a max number not yet implemented");

}

############################################
# Pack by days
############################################
sub packbydays {
# Pack message directories by keeping only those message entries
# up to opt_d days old.  Defaults to # of days set for msg area
# in areasbbs table;  if that is set to 0 & opt_d is not defined
# then no packing is done.

    my ($sql_stmt, $daysold, $count, $dbh);

    if ($opt_d) {
	$daysold = $opt_d;
    } else 
    # this will later check the areasbbs table entry for the maxdays field,
    # if that is not zero, then $daysold gets set to it.  if zero, return.
        &logging($logfile, $progid, "Can't pack $PACKAREA; age not set ");
	return;
    }

    # assumes $PACKAREA variable holds area name 
    &logging($logfile, $progid, "Packing $PACKAREA by age");

#   add code check if area entry is present first ?
    $sql_stmt = "DELETE FROM $PACKAREA ";
    $sql_stmt .= "WHERE TO_DAYS(NOW()) - TO_DAYS(datetime) => $daysold ";

    if ($DEBUG) &logging($logfile, $progid, "$sql_stmt");
    #
    $count = $dbh->do( "$sql_stmt " )
	or die &logging($logfile, $progid, $dbh->errstr); 
    
    &logging($logfile, $progid, "$count messages deleted from $PACKAREA");

}

############################################
# findhighmsg
############################################

#- rewrite findhighmsg subroutine to actually find the highest numbered message,
#regardless of how many msgs are in the message table or if those messages 
#are all numbered sequencially.

sub findhighmsg {
#
    &logging($logfile, $progid, "findhighmsg subroutine not yet implemented");
	
}

############################################
# Help
############################################
sub help {
    print "\nUsage:  bbsdbpladm.pl [-p max|age] [-n max] [-d days] [-l logfile] [-x] \n\n";
    print "      -p   'max' or 'age'; defaults to no packing\n";
    print "      -n   Pack messages by max number of messages (defaults to 100) \n";
    print "      -d   Pack messages by numbers of days to keep (defaults to 7)\n";
    print "      -r   Renumber text message files, starting them at 1.\n";
    print "      -a   Name of area to pack, defaults to 'all'\n";
    print "      -b   Base of message text file directories (defaults to 'msgs')\n";
    print "      -l   Log filename (defaults to bbsdbpl.log)\n";
    print "\n";
}

############################################
# openftndb
############################################
sub openftndb {
# Open message database
#   Assumes that $dbname, $dbuser, & $dbpass
# have been set.  $dbuser must already 
# have the priveledges to create a table.

    if ($opt_v) &logging($logfile, $progid, "Opening Msg database");
	
    my $dbname = 'ftndbtst';
    my $dbuser = 'sysop';
    my $dbpass = 'ftntstpw';
 
    use DBI;

    ( $dbh = DBI->connect("dbi:mysql:dbname=$dbname", $dbuser, $dbpass) )
	or die &logging($logfile, $progid, $DBI::errstr);

}

############################################
# closeftndb
############################################
sub closeftndb {
#
    if ($opt_v) &logging($logfile, $progid, "Closing Msg database");

    ( $dbh->disconnect )
	or die &logging($logfile, $progid, $DBI::errstr);
	
}

############################################
