#!/usr/bin/perl -w
#
# pkt2bbs.pl - v0.1 
# This is a script to toss messages from inbound fidonet packets
# to bbs message area tables in an existing MySQL database. 
#  Copyright (c) 2001-2002 Robert James Clay.  All Rights Reserved.
#  This is free software;  you can redistribute it and/or
#  modify it under the same terms as Perl itself.

# Setup

use strict;
use File::Copy;
use Getopt::Std;
use vars qw/ $opt_i $opt_l $opt_t $opt_k $opt_v  $opt_x $opt_h/;

use FTN::Utility qw(&logging);

my ($dbh, $sql_stmt, $attrib, $area, $datetime, $fromnode, $tonode, $from, 
    $to, $subj, $msgbody, $msgid, $replyid, $Logfile, $FILE,
    $cost, $osep, $orgnet, $destnet, $sec, $min, $hour, $mday, $mon, $year,

    $wday, $s, $k, $i, $DEBUG, $TYPE, $PKT, $KEEPPKT, $INDIR, $FILENAME, $EXT, $UNARC,
#    $PKTver, $orgnode, $destnode, $seenby, $mailer, $origin, $buf, $s, $k, $i,
    $seenby, $mailer, $origin,

    @lines, @moy, @wdy, @time, @files, @args, @kludges);

my $TEMPDIR = "/tmp/pkt2bbs.$$";

# Archivers
# my $ZIP = "/usr/bin/zip";

# Unarchiver
my $UNZIP = "/usr/bin/unzip";

# No user serviceable parts inside

&commandline();

&checkformail();

&expandmail();

&tossmail();

&cleanup();

############################################
# subroutines
############################################
# Help
############################################
sub help {
    print "\nSupported command line arquments:\n\n";
    print "-i	Inbound dir & path\n";
    print "-l	Log File & path\n";
    print "-t	Toss Mail\n";
    print "-k	Keep pkts\n";
    print "-x	Debug Mode\n";
    print "\n";
}

############################################
# Check for command line arguments, handle.
############################################
# Add handlers for other command line arguments here
############################################
sub commandline {

    getopts('i:l:htkxv');


    if ($opt_v) {
	&log("Verbose flag is set ...");
    }
	
    if ($opt_x) {
        $DEBUG = $opt_x;	# set debug flag
        undef $opt_x;
        print "\nDebug flag is set ...\n";
    } else {
        $DEBUG = 0;
    }
	
    if ($opt_i) {
        $INDIR = $opt_i;   # set inbound directory & path
        undef $opt_i;
    } else {
        &help ();	#   exit after displaying help if	
	exit(1);	# no inbound directory specified
    }
	
    if ($opt_l) {
        $Logfile = $opt_l;	# set log path 
        undef $opt_l;
    } else {
        $Logfile = "pkt2bbs.log";  # in current dir
    }

    if ($opt_h) {
        &help ();
	exit(1);
    }

    if ($opt_k) {
        $KEEPPKT = $opt_k;	# set keep packets flag
        undef $opt_k;
        print "\nKeeping FTN packets...\n";
    } else {
        $KEEPPKT = 0;
    }
	
    if ($opt_t) {
        print "\nContinueing...\n";
    }
}

############################################
# Check for new mail
############################################
sub checkformail {
    if ($DEBUG) {&log("Proccessing Dir:  $INDIR...")};	# for testing
    opendir(DIR, $INDIR);
    @files = grep(/.*\.s[ua][0-9a-z]|mo[0-9a-z]|t[hu][0-9a-z]|we[0-9a-z]|fr[0-9a-z]|pkt/i, readdir(DIR));
    closedir(DIR);
    if ($DEBUG) {print @files};
    exit if ($#files == -1);
}

############################################
# If new mail archives, expand
############################################
sub expandmail {
    # Unarchive mail.
	
    if ($DEBUG) {&log("Unarchiving...")};	# for testing
	
    if ( ! -d "$TEMPDIR" ) {mkdir ("$TEMPDIR", 0750 )};

    if ($DEBUG) {print @files};

    foreach $FILE (sort @files) {
		
	# if $FILE = "." or "..", do next file

	if ($DEBUG) {&log("Processing File:  $INDIR/$FILE")};
		
	($FILENAME, $EXT) = split (/\./, "$FILE");
		
	undef $FILENAME;
				
	if ( "$EXT" ne "pkt" ) {
	    open ARCHIVE, "$INDIR/$FILE" or die "Unable to open $FILE: $!\n";
	    read ARCHIVE, $TYPE, 2;
				
	    if ($TYPE eq "PK") {
		@args = ("$UNZIP", "-q", "$INDIR/$FILE", "-d $TEMPDIR");
	    }
	    else {
	        # Default unarchiver
		@args = ("$UNARC", "x", "$INDIR/$FILE");
	    }
			
	    !system "@args" or die "@args failed: $?";
			
	    close ARCHIVE;
	}
	else {
	    if ( ! -d "$TEMPDIR" ) {
	    mkdir ("$TEMPDIR", 0750 );
	    }
			
	    copy("$INDIR/$FILE", "$TEMPDIR/$FILE");
        }
		
        # keep packets if flag is set?
	if (!$KEEPPKT) {
        unlink ("$INDIR/$FILE");
	}
		
    }
}

############################################
# Toss expanded packets
############################################
sub tossmail {
    # Toss mail
	
    if ($DEBUG) {&log("Tossing...")};	# for testing
	
    opendir(TEMPDIR, $TEMPDIR) or die "Could not open $TEMPDIR.\n";
#    my @PKTS = readdir(TEMPDIR);   # filter this for just *pkt/*PKT files
    my @PKTS = grep(/.*\.pkt/i, readdir(TEMPDIR));
    closedir(TEMPDIR);
	
    # connect to database
    &openftndb;
    
    if ($DEBUG) {&log("About to start tossing")};
    #  process packets

    foreach $PKT (sort @PKTS) {    
        if ($DEBUG) {&log("Toss messages here")};
	print "$TEMPDIR/$PKT\n";
	$PKT = "$TEMPDIR/$PKT";
	open(PKT,$PKT) || &log("$PKT,$!");
	binmode(PKT);
	&log("Open: $PKT"); # change to just for debug
	&readpkt();
	&log("Deleting: $PKT"); #  change to just for debug
	unlink($PKT); 
    }

    if ($DEBUG) {&log("Completed  tossing")};

    # disconnect from database
    &closeftndb;
#   ( $dbh->disconnect )
#       or  die $DBI::errstr;
}

###############################################
# Post New Messages from packet to message base
###############################################
sub readpkt {

    my ($PKTver,$orgnode,$destnode,$orgnet,$destnet,$attrib,$cost,
    $buf);
#    $buf,$origin,$mailer,$seenby);

#    local($to,$from,$subj); #,$msgid);
#    local($buf,$s,$osep); 

    read(PKT,$buf,58);  

    while (!eof(PKT))
    {
          last if (read(PKT,$buf,14) != 14);
          ($PKTver,$orgnode,$destnode,$orgnet,$destnet,$attrib,$cost) = unpack("SSSSSSS",$buf);

	  undef $PKTver;		#  not used for anything yet - 8/26/01 rjc
	  undef $attrib;		#  not used for anything yet - 8/26/01 rjc

          $osep = $/;                   
          $/ = "\0";                    

          $datetime = <PKT>;         
          if (length($datetime) > 20) 
          {
             $to = substr($datetime,20);
          } else {
             $to = <PKT>;
          }
          $from = <PKT>;
          $subj = <PKT>;

          $to   =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $to   =~ tr/\0-\037/\040-\100/;     # mask control characters
          $from =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $from =~ tr/\0-\037/\040-\100/;     # mask control characters
          $subj =~ tr/\0-\037/\040-\100/;     # mask control characters

          $s = <PKT>;      
          $/ = $osep;

          $s =~ s/\x8d/\r/g;    
          @lines = split(/\r/,$s);  

          undef $s;     

          next if ($#lines < 0);      

          $area = shift(@lines);          
          $_=$area;
          $area="NETMAIL" if /\//i;		# default netmail area name
          $area =~ s/.*://;			# strip "area:"
          $area =~ tr/a-z/A-Z/;			# Force upper case ???
          

          for ($i = $k = 0; $i <= $#lines; $i++) 
          {
               if ($lines[$i] =~ /^\001/) 
               {       
                  $kludges[$k++] = splice(@lines,$i,1);
                  redo;
               }
          }
          
    for (;;) {
              $_ = pop(@lines);
              last if ($_ eq "");  
          if (/ \* origin: /i) 
          {
             $origin = substr($_,11);
             last;
          }
          if (/---/) 
          {
             $mailer = $_;
          }
          if (/seen-by/i) 
          {
            $seenby=$_;
          }
    }
      
      
    if ($#lines < 0)           
    {
       @lines = ("[empty message]");
    }
    
    # build msg body
    &buildmsgbody;

    # get msgid & replyid from kludges?
    $msgid = "not avail";
    $replyid = "not avail";
            
    # Save msg to database, to a table = $area
        
    $fromnode =  "1:$orgnet/$orgnode\n";   #   need to pull zone #'s from
    $tonode = "1:$destnet/$destnode\n";	   # msg instead of defaulting 1 
    
    $sql_stmt = "INSERT INTO ";
    $sql_stmt .= $area;	#  this assumes that there is one with this name
    $sql_stmt .= "(ftscdate,fromnode,tonode,fromname,toname,";
    $sql_stmt .= "subject,msgid,replyid,local,body) ";
    $sql_stmt .= "VALUES (";
    $sql_stmt .= "'$datetime', ";
      # removed this: $tz\n";  		# not useing this yet
    undef $cost;				# not useing this yet...
    $sql_stmt .= "'$fromnode', ";
    $sql_stmt .= "'$tonode', ";
    $sql_stmt .= "'$from', ";   
    $sql_stmt .= "'$to', ";
    $sql_stmt .= "'$subj', ";

# need to pull this info from the kludges?
    $sql_stmt .= "'$msgid', ";   
    $sql_stmt .= "'$replyid', ";  

    $sql_stmt .= "'N', ";	# N = non-local msg

#   msg body then added to sql_statment using DBI quote function
    $sql_stmt .= $dbh->quote($msgbody);
    $sql_stmt .= ") ";

#   print " $sql_stmt ";
    
    $dbh->do( "$sql_stmt " );	# execute sql statement, adding msg to db

    }   # end while
}   # end sub readpkt

############################################
# Build message body
############################################

sub buildmsgbody {

#	undef $msgbody;	#  ensure that it's empty each time?
	$msgbody="";	#  ensure that it's empty each time?

	foreach $s (@lines) 
	{
		$s =~ tr/\0-\037/\040-\100/;   
		$s =~ s/\s+$//;                
		$s=~tr/^\*/ /;
		$msgbody .= "$s\n"; 
	}

	$msgbody .= "$mailer\n" if ($mailer);
	$msgbody .= " * Origin: $origin\n";
	$msgbody .= "$seenby\n" if ($seenby);

	foreach $s (@kludges)
	{
		$msgbody .= "$s\n";
	}    
}

############################################
#  Datedime
############################################
sub datetime {
#    local($sec, $min, $hour, $mday, $mon, $year, $wday);  #  don't need?
    @moy = ('jan','feb','mar','apr','may','jun',
            'jul','aug','sep','oct','nov','dec');
    @wdy = ('mon','tue','wed','thu','fri','say','sun');
    ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime();
    sprintf("%s, %d %s %d %02d:%02d:%02d",
            $wdy[$wday], $mday, $moy[$mon], $year, $hour, $min, $sec);
}

#############################################
#  logging subrouting.  requires FTN::Utility
#############################################
sub log {
#
    my(@text) = @_;
    my $progid="RPKT";

    &logging($Logfile, $progid, @text);

}

############################################
# open FTN MySQL database for operations
############################################
sub openftndb {
# Open message database
#   Assumes that $dbname, $dbuser, & $dbpass
# have been set.  $dbuser must already 
# have the priveledges to create a table.

    if ($opt_v) {&log("Opening FTN database")};
	
    my $dbname = 'ftndbtst';
    my $dbuser = 'sysop';
    my $dbpass = 'ftntstpw';
 
    use DBI;

    ( $dbh = DBI->connect("dbi:mysql:dbname=$dbname", $dbuser, $dbpass) )
	or die &log($DBI::errstr);

}

############################################
# Close FTN 
############################################
sub closeftndb {
#
    if ($opt_v) {&log("Closing FTN database")};

    ( $dbh->disconnect )
	or die &log($DBI::errstr);
	
}

###########################################################
#  do cleanup
###########################################################
sub cleanup {

    if ( -d $TEMPDIR ) {
    	rmdir $TEMPDIR or die "Unable to remove $TEMPDIR: $!\n";
    }
}

###########################################################

